-- A state machine
import "CoreLibs/object"

class("StateMachine").extends()

function StateMachine.new(params, stateTable)
   local fsm = StateMachine(params, stateTable)
   return fsm
end

-- params is a table of parameters to always pass to the functions
-- stateTable is a table of initial states. Entries should be of the form:
--   name = {main=mainFunc, transition=transFunc}
--   where mainFunc and transFunc are functions that should be executed in relation to the state.
--   the State Machine will call mainFunc on execute()
--   the State Machine will call transFunc when the state is entered
function StateMachine:init(params, stateTable)
   self.currentState = nil
   self.parameters = params or {}
   self.states = stateTable or {}
end

function StateMachine:addState(name, mainFunc, transFunc)
   self.states[name] = {main=mainFunc, transition=transFunc}
end

-- execute the main function for the current state.
-- returns true if the function was executed, false otherwise
function StateMachine:execute()
   if self.currentState == nil or not self.states[self.currentState] then
      return false
   end

   self.states[self.currentState].main(table.unpack(self.parameters))
   return true
end

-- change to the specified state. returns true if the state change is successful
function StateMachine:changeState(state)
   if not self.states[state] then return false end

   self.currentState = state
   if self.states[state].transition then
      self.states[state].transition(table.unpack(self.parameters))
   end
   return true
end
